'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
const vscode = require("vscode");
const process = require("child_process");
const macro = require("./macro");
//import { close } from 'fs';
const clipboardy = require("clipboardy");
// We only want to spawn (execute) once for these commands (calc tools).
let boxAddDel = false;
let chkBoxCtrl = false;
let blanketyBlank = false;
let boxNum = false;
let codeInsp = false;
let makeComment = false;
let calcHelper = false;
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    // Use the console to output diagnostic information (console.log) and errors (console.error)
    // This line of code will only be executed once when your extension is activated
    console.log('Congratulations, your extension "calc-tools-ext" is now active!');
    // The command has been defined in the package.json file
    // Now provide the implementation of the command with  registerCommand
    // The commandId parameter must match the command field in package.json
    let commands = vscode.extensions.getExtension('STF.calc-tools-ext').packageJSON.contributes.commands;
    for (let command of commands) {
        let disposable = vscode.commands.registerCommand(command.command, () => {
            // The code you place here will be executed every time your command is executed
            executeCommand(vscode.workspace.getConfiguration(command.command));
        });
        context.subscriptions.push(disposable);
    }
}
exports.activate = activate;
function executeCommand(config) {
    let command = config.get('command');
    let args = config.get('args', []);
    let cwd = config.get('cwd');
    if (!command) {
        vscode.debug.activeDebugConsole.appendLine('Error!: null command');
        vscode.window.showErrorMessage('null command!');
        return;
    }
    if (vscode.window.activeTextEditor === undefined) {
        vscode.window.showInformationMessage("Calc tools will not be launched from the current editor.");
        return;
    }
    let fileName = vscode.window.activeTextEditor.document.fileName.toLowerCase();
    if (!(/\.(rbw?|js)$/.test(fileName))) {
        //vscode.window.showInformationMessage("Calc tools only launch from Ruby (.rb|.rbw) or JavaScript (.js) documents.")
        vscode.window.showInformationMessage(`Bad File Name: ${fileName}`);
        return;
    }
    let commandFile = command.replace(/^.*[\\\/]/, '');
    try {
        // First check if calc tool is already running, we return if so, else
        // check for calc tools that will read from the clipboard after launching.
        switch (commandFile) {
            // This handles 'Current Date Comment' and 'No Instructions' comments.
            // Slim chance that we would need this, only if an error window is being
            // displayed by MakeComment.exe and someone executes one of these commands
            // again.
            case "MakeComment.exe": {
                if (makeComment) {
                    vscode.window.showInformationMessage("MakeComment is already running.");
                    return;
                }
                makeComment = true;
                break;
            }
            case "BoxAddsDeletes.exe": {
                if (boxAddDel) {
                    vscode.window.showInformationMessage("Box Adds/Deletes is already running.");
                    return;
                }
                boxAddDel = true;
                break;
            }
            case "ChkBoxCtrl.exe": {
                if (chkBoxCtrl) {
                    vscode.window.showInformationMessage("Check Box Control is already running.");
                    return;
                }
                chkBoxCtrl = true;
                toClipboardPreCommand();
                break;
            }
            case "BlanketyBlank.exe": {
                if (blanketyBlank) {
                    vscode.window.showInformationMessage("Blankety Blank is already running.");
                    return;
                }
                blanketyBlank = true;
                toClipboardPreCommand();
                break;
            }
            case "BoxNum.exe": {
                if (boxNum) {
                    vscode.window.showInformationMessage("Box Num/Code Replicator is already running.");
                    return;
                }
                boxNum = true;
                toClipboardPreCommand();
                break;
            }
            case "CodeInspector.exe": {
                if (codeInsp) {
                    vscode.window.showInformationMessage("Code Inspector is already running.");
                    return;
                }
                codeInsp = true;
                break;
            }
            case "CalcHelper.exe": {
                if (calcHelper) {
                    vscode.window.showInformationMessage("Calc Helper is already running.");
                    return;
                }
                calcHelper = true;
                break;
            }
        }
    }
    catch (err) {
        vscode.debug.activeDebugConsole.appendLine(err);
        vscode.window.showErrorMessage(err);
        return;
    }
    macro.create().then((obj) => {
        //vscode.window.showInformationMessage(`Entered macro.create() for: ${commandFile}`);
        //vscode.debug.activeDebugConsole.appendLine(`Entered macro.create() for: ${commandFile}`);
        let replacedArgs = args.map((arg) => {
            return obj.replace(arg);
        });
        let replacedCwd = obj.replace(cwd) || obj.ProjectDir || obj.ItemDir;
        //vscode.window.showInformationMessage(`In macro.create() - replacedArgs[0]: ${replacedArgs[0]}`);
        //vscode.debug.activeDebugConsole.appendLine(`In macro.create() - replacedArgs[0]: ${replacedArgs[0]}`);
        /*
        There will not be an immediate switch over to JS, so we will be doing mostly Ruby
        code editing and 'some' JS. Decided to park most of the tools in a separate
        Ruby folder off from the Calc Tools folder as these have code which handles
        startup from the SciTE editor and will need startup code to handle this editor
        which is different. The JS versions will have the SciTE code removed and some tools
        will need tweaking in other areas.

        Check if tool is being launched from a Ruby or a Javascript document, args[0]
        holds the file name and path.
        */
        let lang = replacedArgs[0].split('.').pop().toUpperCase();
        //vscode.window.showInformationMessage(`In macro.create() - File Extension: ${lang}`);
        //vscode.debug.activeDebugConsole.appendLine(`In macro.create() - File Extension: ${lang}`);
        if ((lang === "RBW") || (lang === "RB")) {
            switch (commandFile) {
                case "ChkBoxCtrl.exe": {
                    command = "\\\\PDC\\FormsProgramming\\Shared Code\\Calc Tools\\Ruby\\ChkBoxCtrl.exe";
                    break;
                }
                case "BlanketyBlank.exe": {
                    command = "rubyw.exe";
                    replacedArgs.unshift("\\\\PDC\\FormsProgramming\\Shared Code\\Calc Tools\\Ruby\\tk_blankety_blank_ver3.rbw");
                    break;
                }
                case "CodeInspector.exe": {
                    command = "\\\\PDC\\FormsProgramming\\Shared Code\\Calc Tools\\Ruby\\CodeInspector.exe";
                    break;
                }
            }
        }
        try {
            let child = process.spawn(command, replacedArgs, {
                cwd: replacedCwd
            });
            child.on('exit', function (code) {
                if (code === null) {
                    vscode.debug.activeDebugConsole.appendLine(`${commandFile} process terminated abnormally.`);
                    vscode.window.showErrorMessage(`${commandFile} process terminated abnormally.`);
                }
                else {
                    //console.log(`${commandFile} process terminated normally with exit code: ${code}`);
                    if (commandFile === "MakeComment.exe") {
                        // This executible [user does not see it] writes to the clipboard
                        // so we need to read it here after the spawn.
                        // Also: For some reason I need to reset this flag here [covers if
                        // someone tries to run it again while an error window is up].
                        makeComment = false;
                        obj.Clipboard = getClipboardPostCommand();
                        vscode.window.activeTextEditor.edit(edit => edit.insert(vscode.window.activeTextEditor.selection.end, obj.Clipboard));
                    }
                }
                // Regardless of the exit code we want to do this, they were preset above:
                resetCommandFlags(commandFile);
            });
            child.on('error', function (err) {
                if (commandFile === "MakeComment.exe")
                    makeComment = false;
                // This may be called twice, but I don't think it will cause any harm.
                resetCommandFlags(commandFile);
                vscode.debug.activeDebugConsole.appendLine(`Error launching ${commandFile}: ${err}`);
                vscode.window.showErrorMessage(`Error launching ${commandFile}: ${err}`);
            });
        }
        catch (err) {
            vscode.debug.activeDebugConsole.appendLine(err);
            vscode.window.showErrorMessage(err);
        }
    });
}
function resetCommandFlags(cmdFile) {
    switch (cmdFile) {
        // Leaving "MakeComment.exe" case here for paranoia reasons.
        case "MakeComment.exe": {
            makeComment = false;
            break;
        }
        case "BoxAddsDeletes.exe": {
            boxAddDel = false;
            break;
        }
        case "ChkBoxCtrl.exe": {
            chkBoxCtrl = false;
            break;
        }
        case "BlanketyBlank.exe": {
            blanketyBlank = false;
            break;
        }
        case "BoxNum.exe": {
            boxNum = false;
            break;
        }
        case "CodeInspector.exe": {
            codeInsp = false;
            break;
        }
        case "CalcHelper.exe": {
            calcHelper = false;
            break;
        }
    }
}
;
// We call this after executing commands that change the clipboard and we want that info.
// The original author of this extension allowed for the passing of the clipboard contents
// to an external command. See macro.ts.
function getClipboardPostCommand() {
    return clipboardy.readSync();
}
// We call this before executing commands that call applications which read from the
// clipboard upon startup from the Viaual Studio Code editor.
function toClipboardPreCommand() {
    if (vscode.window.activeTextEditor !== undefined) {
        let selection = vscode.window.activeTextEditor.selection;
        clipboardy.writeSync(vscode.window.activeTextEditor.document.getText(selection));
    }
}
// this method is called when your extension is deactivated
function deactivate() {
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map